function [P, Q] = ranksvm_joint_feat_sel(feat, trainSet, par)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Developed by Lin Chen and Baoxin Li, and documented in 08/2014.
% This package is % an experiment version and the code has not been optimized. 
% This is based on the following publication:
%    L. Chen, Q. Zhang, and B. Li, Predicting Multiple Attributes via Relative Multi-task Learning, 
%      IEEE Conference on Computer Vision & Pattern Recognition (CVPR), June, 2014.
%
% You are free to use the code for your research provided:
%    (1) You do not redistribute the code without the authors' explicit permission;
%  and 
%    (2) You clearly cite the source of the work (i.e., the above publication).
%
%
% To use this package, the user needs to have the package MALSAR installed 
% (http://www.public.asu.edu/~jye02/Software/MALSAR/)
%
% [P, Q] = ranksvm_joint_feat_sel(feat, trainSet, par)
% 
% Input:
% feat - the training inputs /feature vectors and is an m x n x d 
%        matrix, m is the dimensionality of each point, n is the 
%        number of points, d is the number of learning tasks.
%
% trainPairSet contains two cells.
% trainPairSet{1,i} - a sparse p x n x d matrix O represents the 
%     preference ordered set, where p is the number of prefrence 
%     ordered pairs. Each row of O should contain exactly one +1 
%     and one -1. If the p-th preference pair says i > j (strength 
%     of attribute in image i is greater than strength of attribute
%     in image j) then O(p,i) = 1 and O(p,j) = -1;
% trainPairSet{2,i} - a similar p x n x d matrix S represents the 
%     unordered set. If the p-th preference pair says i and j are 
%     similar, then either S(p, i) or S(p, j) is 1 and the other is -1.
%
% par.rho1/rho2 - super parameters for quadratic optimization;
% par.mu1/mu2 - super parameters for rMTFL by MALSAR
% par.lambda - controls the convergence speed
% par.max_it - maximal iteration times
%
% Output:
% P - the matrix corresponding to selected features
% Q - detected outlier
% W=P+Q - the learned projection model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% parameter check
if ~isfield(par, 'rho1')
    par.rho1 = 1;
end
if ~isfield(par, 'rho2')
    par.rho2 = 1;
end
if ~isfield(par, 'mu1')
    par.mu1 = 1;
end
if ~isfield(par, 'mu2')
    par.mu2 = 1;
end
if ~isfield(par, 'lambda')
    par.lambda = 500;
end
if ~isfield(par, 'max_it')
    par.max_it = 500;
end

taskN = size(trainSet, 2);
featDim = size(feat, 1);

%% construct training feature set based on the training label
trainFeat = cell(2, taskN); % the first/second row is ordered/unordered pair set
for i=1:taskN
    priFeat = [];
    pairs = trainSet{1,i};
    for j=1:size(pairs,1)
        priFeat = [priFeat, feat(:,pairs(j,:)==1,i)-feat(:,pairs(j,:)==-1,i)];
    end
    trainFeat{1,i} = priFeat;
    
    simFeat = [];
    pairs = trainSet{2,i};
    for j=1:size(pairs,1)
        simFeat = [simFeat, feat(:,pairs(j,:)==1,i)-feat(:,pairs(j,:)==-1,i)];
    end
    trainFeat{2,i} = simFeat;
end

%% parameter initialization
% group lasso parameters
opts.init = 0; % guess start point from data.
opts.tFlag = 1; % terminate after relative objective value does not changes much.
opts.tol = 10^-6; % tolerance.
opts.max_it = 500; % maximum iteration number of optimization.

% Quadratic optimization parameters
opts1 = optimset('Algorithm','interior-point-convex','MaxIter',500);

%% construct quadratic optimization formulation
featSet = cell(2,taskN);
totalTrainSize = 0;
for i=1:taskN
    featSet{1,i} = [trainFeat{1,i}, trainFeat{2,i}];
    featSet{2,i} = [size(trainFeat{1,i},2), size(trainFeat{2,i},2)];
    totalTrainSize = totalTrainSize+size(trainFeat{1,i},2)+size(trainFeat{2,i},2);
end
Y = zeros(featDim*taskN, totalTrainSize);
sp = 1;
a = [];
b1 = [];
b2 = [];
for i=1:taskN
    Esize = size(trainFeat{1,i},2); % number of ordered pairs of task i
    Fsize = size(trainFeat{2,i},2); % number of un-ordered pairs of task i
    offset = Esize+Fsize;
    Y((i-1)*featDim+1:i*featDim,sp:sp+offset-1) = featSet{1,i};
    sp = sp+offset;
    a = [a; ones(Esize,1); zeros(Fsize,1)];
    b1 = [b1; par.rho1*ones(Esize,1); par.rho2*ones(Fsize,1)];
    b2 = [b2; zeros(Esize,1); par.rho2*ones(Fsize,1)];
end
H=Y'*Y;
A = [eye(size(H,1)); -eye(size(H,1))];
b = [b1; b2];

%% solve through iteration
odst = 10e-10;
delta = 10e10;
thresh = 10e-10;
it = 1;
lambda = 0.05*par.lambda;
Wt = rand(featDim*taskN,1);
while delta>thresh && it<opts.max_it
    %% multi-task feature selection through rMTFL
    X = cell(taskN, 1);
    Y = cell(taskN, 1);
    for i=1:taskN
        X{i} = feat(:,:,i)';
        Y{i} = transpose(Wt(featDim*(i-1)+1:featDim*i)'*feat(:,:,i));
    end
    [W,~,P,Q] = Least_rMTFL(X, Y, par.mu1, par.mu2, opts);
    
    %% quadratic optimization solve equ. 5
    w=[];
    for i=1:taskN
        w=[w;transpose(W(:,i)'*featSet{1,i})];
    end
    f = lambda*w-(1+lambda)*a;
    x = quadprog(H,f,A,b,[],[],[],[],[],opts1);
    Wt = zeros(featDim, taskN);
    sp = 1;
    for i=1:taskN
        offSet = size(featSet{1,i},2);
        Wt(:,i)=(1/(1+lambda))*(lambda*W(:,i)+featSet{1,i}*x(sp:sp+offSet-1,1));
        sp = sp+offSet;
    end
    
    %% calculate difference
    dst = 0;
    for i=1:taskN
        dst = dst + norm(Wt(:,i)'*feat(i)-W(:,i)'*feat(i),'fro');
    end
    delta = abs((dst-odst)/odst);
    odst = dst;
    it = it+1;
    Wt = reshape(Wt,[featDim*taskN, 1]);
    
    %% update weight
    if lambda<par.lambda
        lambda = lambda+0.05*par.lambda;
    end
end

end